local addonName = ...
local Master = LibStub('AceAddon-3.0'):GetAddon("Dominos")
local Addon = Master:CreateClass('Frame', Master.Frame)
Addon.NAME = addonName
_G[addonName] = Addon


Addon.Config = "Dominos-Config"

Addon.master = Master
Addon.lib = LibStub('LibSharedMedia-3.0', true)

Addon.frames = {}

local protoFrames = {}

function Addon:New(name)
	local name = string.lower(name)
	local f = self:Bind(self.proto.New(self, name))
	protoFrames[name] = f
	f.container = f:CreateTexture(nil, 'ARTWORK')
	f.container:SetPoint("Center", f)
	f:Reload()
	--f:SetParent(self.header)
	Addon.frames[name] = f
	return f
end


function Addon:GetDefaults()
	local def
	if self and self.id and self.defaults[self.id] then 
		def = Addon.Copy(Addon.defaults[self.id])
	else
		def = Addon.Copy(Addon.baseDefaults)
		def.x = 0
		def.y = 0
		def.point = "Center"
	end
	return def
end

function Addon:Reload()
	if self.Load then
		self:Load()
	end
	self:SetParent(self.parent or self:GetParent())
	self:Layout()
	self:Rescale()
	self:Reposition()
	self:Reanchor()
	if self.UpdateWidgets then
		self:UpdateWidgets()
	end
end

function Addon:Delete()
--	Addon.master.db.profile.frames[self.id] = nil
--	self:Free()
end

function Addon:Layout()
	if self.UpdateWidgets then
		self:UpdateWidgets()
	else
		local pad = self:GetPadding()
		local w, h = self.sets.width, self.sets.height
		self:SetSize(w + pad, h + pad)
		self.container:SetPoint("Center", self)
		self.container:SetSize(w,h)
		self.sets.frameStrata = self.sets.frameStrata or 1
		self.sets.frameLayer = self.sets.frameLayer or 1
		local lay = Addon.layers[self.sets.frameStrata]
		self:SetFrameStrata(lay)
		self:SetFrameLevel(self.sets.frameLayer)
	end
end


local mod = Addon.master:NewModule('Frame')

local enabledFrames = {}

function mod:OnInitialize()
	Addon.master.db.profile.units = Addon.master.db.profile.units or defaults
	
	if Dominos_UnitsVersion ~= 6 then
		Dominos_UnitsVersion = 6
			Addon.master.db.profile.units = defaults
			for i,b in pairs(Addon.master.db.profile.frames) do
				if Addon.IsUnitValid(i) then
					Addon.master.db.profile.frames[i] = nil
				end
			end
		--reset all settings!
		self:Load()
	end
	
end

--[[

local advanced = {}

for widget, settings in pairs(target)do
	if type(settings) == "table" then
		advanced[widget] = {}
		for i, b in pairs(settings) do
			if type(b) == "table" then
				advanced[widget][i] = b
			end
		end
	else
		advanced[widget] = settings
	end
end


--]]




function mod:Load()
	wipe(enabledFrames)

	Addon.frames = {}
	for unit, b in pairs(Addon.getSets().frames) do
		if b == true then
			unit = string.lower(unit)
			local frame = Addon:New(unit)
			frame:Reload()
			tinsert(enabledFrames, frame)
		end
	end

	for frameName, b in pairs(Addon.getSets().hide) do
		if frameName and _G[frameName] then
			_G[frameName]:SetParent(Addon.HIDER)
		end
	end
end

Addon.MasterLoad = mod.Load

function mod:Unload()
--
	if enabledFrames then
		for unit, frame in pairs(Addon.frames) do
			frame:Hide()
			frame:Free()
		end
	end
	wipe(enabledFrames)
end

Addon.MasterUnload = mod.Unload