local Addon = _G[...]

local Menu = {}
Addon.Menu = Menu

local function layout(page)
	page.checkbuttonBox = page.checkbuttonBox or CreateFrame("Frame", nil, page)
	page.checkbuttonBox:SetSize(10, 10)
	page.checkbuttonBox:SetPoint("TopLeft", page, 10,0)
	page.sliderBox = page.sliderBox or CreateFrame("Frame", nil, page)
	page.sliderBox:SetPoint("TopLeft", page.checkbuttonBox, "BottomLeft", -5,0)
	page.sliderBox:SetSize(10, 10)

	
	for type, toggles in pairs(page.toggles) do
		for i, piece in pairs(toggles) do
			piece:ClearAllPoints()
			if type == "checkbutton" then
				if i ~= 1 then
					piece:SetPoint("TopLeft", toggles[i-1], "BottomLeft", 0, piece.spacing or 2)
					page.checkbuttonBox:SetPoint("BottomRight", piece, 0,-5)
				else
					piece:SetPoint("TOPLEFT", page, 0, 2)
					page.checkbuttonBox:SetPoint("BottomRight", piece, 0,-5)
				end
			elseif type == "slider" then
				piece:SetPoint("Right", page, "Right", -20, 0)
				
				if i ~= 1 then
					piece:SetPoint("TopLeft", toggles[i-1], "BottomLeft", 0, -12)
				else
					piece:SetPoint("TopLeft", page.sliderBox, -6, -10)
				end
			end
		end
	end
end

local layers = {
	"BACKGROUND",
	"LOW",
	"MEDIUM",
	"HIGH",
	"DIALOG",
	"FULLSCREEN",
	"FULLSCREEN_DIALOG",
	"TOOLTIP",
}
Addon.layers = layers

local anchors = {
	'TopLeft',
	'Top',
	'TopRight',
	'Right',
	'BottomRight',
	'Bottom',
	'BottomLeft',
	'Left',
	'Center',
}
Addon.anchors = anchors

local function SetLayer(self)
	local lay = layers[self.sets.frameStrata]
	self:SetFrameStrata(lay)
	self:SetFrameLevel(self.sets.frameLayer)
end

local tabbedPanel = {}

local function newPanel(self)
	local panel = Dominos:GetOptions().Panel:New(self)
	Addon.Merge(tabbedPanel, panel)
	panel.container = panel
	panel.owner = self.owner
	panel.self = self
	panel.tabs = {}
	
	panel:SetScript('OnSizeChanged', function() Dominos:GetOptions():Render(panel) end)
	panel:SetPoint('TopLeft', self, 'TopLeft', 0, -60)
	panel:SetPoint('TopRight', self, 'TopRight', 25, -60)
	panel:SetPoint('Bottom', self, 0, 0)
	return panel
end

local function NewSelector(panel) 
	local panelSelector = Dominos:GetOptions().PanelSelector:New(panel)
	panelSelector:SetPoint('TopLeft', panel, 'TopLeft', 0, 20)
	panelSelector:SetPoint('BottomRight', panel, 'TopRight', -30, 0)
	panelSelector.OnSelect = function(self, id)
		panel:OnShowTab(id or self.id)
	end
	panel.panelSelector = panelSelector	

	return panelSelector
end

function tabbedPanel:New(id)
	self.panelSelector:AddPanel(id)
	local panel = newPanel(self)
	local panelSelector = NewSelector(panel)
	self.panels[id] = panel

	Addon.Merge(Menu, panel)
	
	return panel
end

function tabbedPanel:NewTab(id)
	self.panelSelector:AddPanel(id)
	local panel = Dominos:GetOptions().ScrollablePanel:New(self.panelContainer)
	panel:Hide()
	panel:SetParent(self)
	panel.owner = self.owner
			
	panel:SetPoint('TOPLEFT', self, 'TopLeft', 5, 0)
	panel:SetPoint('BottomRight', self, 'BottomRight', -30, 5)
	
	local container = Dominos:GetOptions().Panel:New(panel)
	container:SetScript('OnSizeChanged', function() Dominos:GetOptions():Render(panel) end)
	container:SetPoint('TOPLEFT', panel, 'TopLeft', 0, 0)
	container:SetPoint('TOPRIGHT', panel, 'TopRight', 0, 0)
	container.owner = self.owner
	panel.viewport:SetScrollChild(container)
	
	container:SetScript("OnMouseWheel", function(_, delta)
		panel.viewport:GetScript("OnMouseWheel")(nil, delta)
	end)
	
	panel.container = container
	self.tabs[id] = panel

	container.owner = self.owner
	container.toggles = container.toggles or {checkbutton = {}, slider = {}}
	
	return panel.container
end

function tabbedPanel:ShowTab(id)
	return self.panelSelector:Select(id)
end

function tabbedPanel:OnShowTab(id)
	if self.tabs then
		for i, tab in pairs(self.tabs) do
			if i == id then
				tab:Show()
			else
				tab:Hide()
			end
		end
	end
end

function tabbedPanel:SetOwner(owner)
	self.owner = self.owner
	for i, tab in pairs(self.tabs or self.panels) do
		tab.container:SetOwner(parent, self.owner)
		tab.container.owner = self.owner
	end
end

local function PrepareOptions(opts)
	local trace = {}
	local options = {}
	local d
	local adv
	
	if not opts then return end
	for i , info in pairs(Addon.Copy(opts)) do
		local title = string.lower(info.panel or i)
		if not trace[title] then
			trace[title] = {}
			if (title == "basic") then
				d = title
				tinsert(options , 1, {title = title, info = trace[title]})
			elseif (title == "advanced") then
				adv = {title = title, info = trace["advanced"]}
			else
				d = d or title
				tinsert(options, {title = title, info = trace[title]})
			end
		end
		if adv then
			tinsert(options, adv)
		end
		tinsert(trace[title], info)
	end
	
	return trace, options, d
end

local function GetRestrict(panel)
	local allow = true
	if panel.owner.GetRestrictions then
		local id, include = panel.owner.GetRestrictions()
		if (include == true) then
			if id ~= string.lower(panel.owner.id) then
				allow = false
			end
		elseif include == false then
			if id == string.lower(panel.owner.id) then
				allow = false
			end
		end
	end
	return allow
end

function Menu.NewMenu(self, menu, key, opts, test)
	self.panels = self.panels or {}
	self.panels[self.owner.id] = self.panels[self.owner.id] or {}
	
	menu.AddTab = tabbedPanel.New
	
	key = string.lower(key)

	self.panels[self.owner.id][key] = self.panels[self.owner.id][key] or menu:AddTab(key)
	local panel = self.panels[self.owner.id][key]

	panel.id = key
	panel.owner = self.owner

	local trace, p, d = PrepareOptions(opts)

	
	local l
	
	panel._tabs = panel._tabs or {}
	if not p then return end
	for i, options in pairs(p) do
		local options = Addon.Copy(options)
		local allow = GetRestrict(panel)
		if allow then
			if not panel._tabs[options.title] then
			local tab = panel:NewTab(options.title)
			l = l or options.title
				tab.id = key
				for name, option in pairs(options.info) do
					local func = Menu[option.kind]
					local o, type = func(tab, option)
					if o then
						tab.toggles[type] = tab.toggles[type] or {}
						tinsert(tab.toggles[type], o)
					end
				end
				layout(tab)
				panel._tabs[options.title] = tab
			end
		end
	end
	
	panel:ShowTab(d)

end

--Widgets

do --Slider
	local function Slider(panel, info)	
		local name, key, min, max, step, p = info.name, info.key, info.min, info.max, info.step, string.lower(info.panel)
		if min < 1 and min > 0 then
			min = min * 100
			max = max * 100
		end
		local owner = panel.owner

		local slider =  panel:NewSlider({
			name = name,
			min = min,
			max = max,
			get = function(self) --Getter
				if panel.sets then
					return panel.sets[key] or max/2 --g(widget, key) or max/2
				else
					return owner.sets[panel.id][key] or max/2
				end
			end,
			set = function(self) --Setter
				local val = ceil(self:GetValue())

				if panel.sets then
					if val ~= panel.sets[key] then
						panel.sets[key] = val--g(widget, key, val)
						panel:Layout()
					end
				else
					owner.sets[panel.id][key] = val
					owner:Layout()
				end
				return val
			end,
		})
		slider.height = 30
		slider:SetWidth(200)
		return slider, "slider"
	end

	Menu.Slider = Slider
end

do --CheckButton
	local function CheckButton(panel, info)
		
		local owner = panel.owner
		local name, key = info.name, info.key
		local c = panel:NewCheckButton{
			name = name,
			get = function() 
				if panel.sets then
					return panel.sets[key]
				else
					return owner.sets[panel.id][key]
				end
			end,
			set = function(self, enable)
				local val = self:GetChecked()

				if panel.sets then
					if val ~= panel.sets[key] then
						panel.sets[key] = val
						panel:Layout()
					end
				else
					owner.sets[panel.id][key] = val
					owner:Layout()
				end
				return val
			end
		}
		c.height = 28
		return c, "checkbutton"
	end
	Menu.CheckButton = CheckButton
end

do --EditBox
	local nextName = Addon:CreateNameGenerator("EditBox")

	local function EditBox(p, info)
		local name, key, setter = info.name, info.key, info.setter
		local editBox = CreateFrame("EditBox", nextName(), p,  "InputBoxTemplate")
		editBox:SetAutoFocus(false)
		local set = CreateFrame("Button", editBox:GetName().."Set", editBox, "UIPanelButtonTemplate")
		editBox.title = editBox:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
		editBox.title:SetPoint("Bottom", editBox, "Top")
		local flip, flop = name, "Set"
		local hold = false
		local isLocked = false
		local function flipText(why)
			local text = flip
			if why == "lock" then 
				isLocked = true
			elseif why == "unlock" then 
				isLocked = false
			end
			if isLocked == true then
				text = flop
			else
				text = flip
			end
			if (why == "OnEnter") then
				if isLocked == true then
					text = flip
				else
					text = flop
				end
			elseif (why == "OnLeave")  then
				if isLocked == true then
					text = flop
				else
					text = flip
				end
			end 
			set:SetText(text)
		end
		set:SetScript("OnEnter", function(self)
			flipText("OnEnter")
		end)
		set:SetScript("OnLeave", function(self)
			flipText("OnLeave")
		end)
		local function setText(self)
			if type(key) == "function" then
				local value, title = key(p.owner)
				self:SetText(value or "")
				if title then
					editBox.title:SetText(title)
				end
			else
				self:SetText(p.owner.sets[p.id][key] or "")
			end
		end
		
		editBox:SetScript("OnShow", function(self)
			setText(self)
		end)
		editBox:SetScript("OnEscapePressed", function(self)
			setText(self)
			self:ClearFocus()
		end)
		editBox:SetScript("OnEnterPressed", function(self)
			local text = self:GetText()
			setter(p.owner, text ~= "" and text or nil)
			setText(self)
			self:ClearFocus()
		end)
		editBox:SetScript("OnEditFocusLost", function(self) self:HighlightText(0, 0) flipText("unlock") end)
		editBox:SetScript("OnEditFocusGained", function(self) self:HighlightText() flipText("lock") end)
		set:SetHeight(20)
		set:SetText(name)
		set:SetScript("OnClick", function(self)
			editBox:GetScript("OnEnterPressed")(editBox)
		end)

		editBox.height = 30
		editBox:SetSize(220, 30)
		set:SetPoint("Left", editBox,"Right")
		return editBox, "slider"
	end

	Menu.EditBox = EditBox
end

do --MediaPanel
	local Master = Addon.master
	local LibSharedMedia = Addon.lib
	local NUM_ITEMS, WIDTH, HEIGHT, OFFSET = 8, 150, 20, 0
	--[[ buttons ]]--
	local mediaButton = Master:CreateClass('Button')
	do
		function mediaButton:New(parent)
			local button = self:Bind(CreateFrame('Button', nil, parent))

			button.BackDrop = CreateFrame("Frame",nil, button)
			button.BackDrop:SetPoint("TopLeft", 50,-12)
			button.BackDrop:SetPoint("BottomRight")
			
			
			--button.BackDrop:SetFrameStrata("HIGH")
			
			button:SetSize(285, HEIGHT + 12)
			button:SetNormalFontObject('GameFontNormal')
			button:SetHighlightFontObject('GameFontHighlight')

			button:SetScript('OnEnter', function(self)
				self:GetFontString():SetTextColor(1,1,1,1)
			end)
			
			button:SetScript('OnMouseWheel', function(self, direction)
				local wheel = button:GetParent():GetScript("OnMouseWheel")

				if wheel then
					wheel(parent, direction)
				end
			end)
			button:SetScript('OnLeave', function(self)
				self:GetFontString():SetTextColor(1,1,0,1)
			end)
			
			button:SetScript("OnClick", function()
				local txt = button:GetText()
				if txt then
					if parent.set then
						parent.set(txt)
						parent:Hide()
					end
				end
			end)
			
			--button:EnableMouse(false)
			return button
		end

		function mediaButton:Set(mType, getFileName)
			local fileName = getFileName()
			local filePath = LibSharedMedia:Fetch(mType, fileName)
			self:SetText(fileName)

			if mType == 'border' then
				self.BackDrop:SetBackdrop({
					edgeFile = filePath,
					edgeSize = 16,
				})
				self:SetNormalFontObject('GameFontNormal')
			elseif (mType == 'background') or (mType == 'statusbar') or (mType == 'castborder') or (mType == 'overlay') then
				self.BackDrop:SetBackdrop({
					bgFile = filePath,
					tile = false,
				})
				self:SetNormalFontObject('GameFontNormal')
			end

			local fs = self:GetFontString()

			if fs then
				if mType == 'font' then
					fs:SetFont(filePath, 12)
					self.BackDrop:SetBackdrop(nil)
				else
					fs:SetFont(LibSharedMedia:Fetch('font', 'Friz Quadrata TT'), 12)
				end
				fs:ClearAllPoints()
				fs:SetJustifyH("LEFT")
				fs:SetPoint("BottomLeft", self.BackDrop, "TopLeft")
				fs:SetPoint("BottomRight", self.BackDrop, "TopRight")
				fs:SetTextColor(1,1,0,1)
			end
			

			self.filepath = filePath
			self:Show()
		end
	end

	--[[ panel ]]--
	local mediaPanel = Master:CreateClass('Frame')
	do
		local nextPanelName = Addon:CreateNameGenerator("MediaButton")

		function mediaPanel:New(name, anchor)
			local n = nextPanelName()
			local panel = self:Bind(CreateFrame('Frame', n, anchor))

			
			
		panel:SetBackdrop{
			bgFile   = [[Interface\ChatFrame\ChatFrameBackground]],
		}
		
		
		
			panel:SetBackdropColor(0, 0, 0, 1)
			
			panel:EnableMouse(true)
			panel:SetFrameLevel(1000)
			-- add close button
			local closeButton = CreateFrame('Button', nil, panel)
			closeButton:SetNormalFontObject('GameFontRedLarge')
			closeButton:SetHighlightFontObject('GameFontHighlightLarge')
			closeButton:SetText('×')
			closeButton:SetSize(closeButton:GetFontString():GetSize())
			closeButton:SetPoint('TOPRIGHT', panel, 0, 0)
			closeButton:SetScript('OnClick', function() panel:Hide() end)
			closeButton:SetFrameLevel(panel:GetFrameLevel() + 1)
			panel.close = closeButton

			-- add title text
			panel.title = panel:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
			panel.title:SetPoint('TOPLEFT', 15, -15)


			panel:CreateScrollArea()
			
			--add list buttons
			panel:CreateButtons()

			panel:SetScript('OnHide', function(self)
				if self.holding then
					self.holding:SetChecked(false)
					self.holding:GetScript('OnShow')(self)
					self.holding = nil
				end
			end)

			-- make panel close on escape
			tinsert(UISpecialFrames, panel:GetName())

			return panel
		end

		function mediaPanel:CreateScrollArea()
			self:SetScript('OnShow', function()
				--self.buttons[1]:SetWidth(WIDTH)
			end)

			self:SetScript('OnHide', function()
				--self.buttons[1]:SetWidth(WIDTH + 20)
			end)

			self:SetScript('OnMouseWheel', function(scroll, direction)
				self:UpdateList(direction)
			end)

			self:SetScript('OnEnter', function()
				self:SetScript('OnUpdate', function()
					for i, button in pairs(self.buttons) do
						if MouseIsOver(button) and button:IsShown() then
							if not button.entered then
								button.entered = true
								button:GetScript('OnEnter')(button)
							end
						else
							if button.entered then
								button.entered = nil
								button:GetScript('OnLeave')(button)
							end
						end
					end
				end)
			end)
			self:SetScript('OnLeave', function()
				self:SetScript('OnUpdate', nil)
			end)
	--Convenient, but the mediaButton's need mouseover interaction.
	-- [[
		self:EnableMouse(false)
		--[[
			self:SetScript('OnMouseUp', function()
				for i, button in pairs(self.buttons) do
					if MouseIsOver(button) and button:IsShown() then
						if self.set then
							print(button:GetText())
							self.set(button:GetText())
							self:Hide()
						end
					end
				end
			end)
	--]]
		end

		function mediaPanel:CreateButtons(mType)
			self.buttons = self.buttons or {}

			self.NUM_ITEMS = floor(self:GetParent():GetHeight()/20) - 2
			self.HEIGHT = self:GetParent():GetHeight()
				for i = 1, self.NUM_ITEMS do
					if not self.buttons[i]then
						local button = mediaButton:New(self)

						if i == 1 then
							button:SetPoint('TOPLEFT', -7, -36)
						else
							button:SetPoint('TOPLEFT', self.buttons[i - 1], 'BOTTOMLEFT', 0, -OFFSET)
						end
						tinsert(self.buttons, button)
					end
				end

			for i,button in pairs(self.buttons) do
				--self:SetFrameStrata("HIGH")


				button:SetFrameLevel(100)
				if i > self.NUM_ITEMS then
					button:Hide()
				else
					button:Show()
				end
			end
				
		end	
			
		function mediaPanel:UpdateList(direction)

			local height, m = 20 + 12, 1
			
			if self.mediatype == "overlay" then
				height, m = 75 + 12, 0
			end
			
			local numButtons = floor(self:GetParent():GetHeight()/height) - m
		
			direction = direction or 0
			self.value = self.value  or 0
			self.value = self.value - direction 

			if self.value < 0 then
				self.value = 0
			elseif self.value > (#self.list -  numButtons) then
				self.value = (#self.list -  numButtons)
			end

			for i = 1, numButtons do
				local button = self.buttons[i]
				local index = i + self.value
				local filePath = LibSharedMedia:Fetch(self.mediatype, self.list[index])
				button:Set(self.mediatype, function() return self.list[index] end)
				button:SetHeight(height)
			end
		end
	end

	do
		local Media = {}

		function Media:Display(anchor, clicked, mediaType, get, set)
			local panel = self:GetOrCreatePanel(anchor)
			local holding = panel.holding

			if holding then
				holding:SetChecked(false)
			end

			if clicked and (holding == clicked) then
				clicked:SetChecked(false)
				panel.holding = nil
				return panel:Hide()
			end

			panel.holding = clicked
			clicked:SetChecked(true)
			panel.owner = anchor.owner
			panel:SetParent(anchor)
			panel.title:SetText(mediaType)
			panel:ClearAllPoints()
			panel:SetPoint('TOPLEFT', anchor:GetParent(), 'TOPLEFT', 0, 0)
			panel:SetPoint('BottomRight', anchor:GetParent(), 'BottomRight', 10, -15)
			panel:SetParent(anchor:GetParent())
			panel.mediatype = string.lower(mediaType)
			--panel:SetFrameStrata("FULLSCREEN")
			panel.list = LibSharedMedia:List(panel.mediatype)
			panel.get = function() return anchor.owner[get](anchor.owner) end
			panel.set = function(...) return anchor.owner[set](anchor.owner, ...) end

			panel:CreateButtons()
			
			panel:UpdateList()
			panel:Show()
			
			return panel
		end

		function Media:GetOrCreatePanel(anchor)
			if not anchor.panel then
				anchor.panel = mediaPanel:New(Addon.NAME..'MediaPanel', anchor)
			end
			return anchor.panel
		end

		local nextButtonName = Addon:CreateNameGenerator("MediaButton")

		function Media:NewMediaButton(parent, name, mediaType, get, set)
			local button = CreateFrame('CheckButton', nextButtonName(), parent)
			button:SetHeight(20)

			button.get = function() return parent.owner[get](parent.owner) end
			button.set = function(...) return parent.owner[set](parent.owner, ...) end

			button.preview = mediaButton:New(button)
			button.preview:SetAllPoints(button)
			button.preview:EnableMouse(false)

			button:SetScript('OnClick', function()
				self:Display(parent, button, mediaType, get, set)
			end)

			 button:SetScript('OnShow', function()
				button.preview:Set(string.lower(mediaType), function() return button.get() end)
				 button:SetWidth(275) --don't like this here, but it glitches otherwise. ~Goranaws
			end)

			--button:SetCheckedTexture(button:GetHighlightTexture():GetTexture())

			button:SetText(name)
			
			local  s = button:GetFontString()
			
			button.Title = button:CreateFontString(nil, 'OVERLAY', 'GameFontNormal')
			button.Title:SetPoint("Left")
			button.Title:SetText(name)
			
			s:ClearAllPoints()
			s:SetPoint("TopLeft", button.Title, "TopRight")
			s:SetPoint("TopRight")
			--button:SetWidth(button:GetTextWidth()+8)
			button:GetFontString():SetPoint("Left", button)
			button.get = function()
				return parent.owner[get](parent.owner)
			end
			button.set = function(...)
				return parent.owner[set](parent.owner, ...)
			end
			
			local prev = parent.checkbutton --I'd like to call it parent.button instead, everywhere. ~Goranaws
			if prev then
				button:SetPoint('TOPLEFT', prev, 'BOTTOMLEFT', 0, -2)
			else
				button:SetPoint('TOPLEFT', 2, -2)
			end

			parent.checkbutton = button
			parent.height = parent.height + 27
			return button
		end

		Addon.MediaPanel = Media
	end
end

do --DropDownMenu
	local nextName = Addon:CreateNameGenerator("DropDownMenu")
	local function _Menu(panel, info)
		local name, key, table, p = info.name, info.key, info.table, string.lower(info.panel or "")
		
		local name, isColor, handler, func
		if type(info) == "string" then
			name = info
		else
			name, key, table = info.name, info.key, info.table
		end
		
		local owner = panel.owner
		
		local button = CreateFrame("Button", nextName(), panel)
		button:SetPoint("Left")
		button:SetSize(265, 16)
		button:SetNormalFontObject('GameFontNormalRight')
		button:SetHighlightFontObject('GameFontHighlightRight')
		button:SetBackdrop{
			bgFile   = [[Interface\ChatFrame\ChatFrameBackground]],
			insets   = {left = -2, right = -2, top = -2, bottom = -2},
		}
		
		
		button:SetHitRectInsets(-5, -5 , -7, -5)
		
		button:SetBackdropColor(0.5, 0.5, 0.5, 0.2)

		button:SetText(name or " ")
		button.title = button:GetFontString() 
		button.title:ClearAllPoints()
		button.title:SetPoint("Left")
		
		button.value = button:CreateFontString(button:GetName() .. "Text", "OVERLAY", "GameFontHighlightSmall")
		button.value:SetPoint("Left", button.title, "Right")
		button.value:SetPoint("Right")
		button.value:SetJustifyH("Right")
		
		button:SetScript("OnShow", function(self)
			if self ~= button then
				return
			end
			button:initialize()
			local text
			if panel.sets then
				text = panel.sets[key]
			else
				text = owner.sets[panel.id][key]
			end
			button.value:SetText(text)
		end)
		button:SetScript("OnHide", function() L_CloseDropDownMenus() end)
		button:SetScript("OnClick", function(self)
			L_UIDropDownMenu_SetAnchor(button, 7, 6, "TopRight", button, "TopRight")
			L_ToggleDropDownMenu(1, nil, button)
			PlaySound(SOUNDKIT.U_CHAT_SCROLL_BUTTON)
			button:SetSize(265, 16)
		end)
		function button:initialize(_, level)
			local info = L_UIDropDownMenu_CreateInfo()
			for i, value in ipairs(table) do
				wipe(info)
				info.text = value
				info.func = function(item, name)
					if panel.sets then
						if name ~= panel.sets[key] then
							panel.sets[key] = name
							panel:Layout()
						end
					else
						owner.sets[panel.id][key] = name
						owner:Layout()
					end
					button.value:SetText(name)
				end
				local checked
				if panel.sets then
				
					checked = panel.sets[key]
				else
					checked = owner.sets[panel.id][key]
				end
				info.checked = (value == checked)
				info.arg1 = value
				L_UIDropDownMenu_AddButton(info)
			end
		end
		
		button:SetFrameLevel(50)

		local prev = panel.lastWidget
		if prev then
			button:SetPoint('TopLeft', panel.lastWidget, 'BottomLeft', 0, -3)
		else
			button:SetPoint('TOPLEFT', 0, -4)
		end
		panel.lastWidget = button
		panel.height = panel.height + 26
		button.height = 26
		button.spacing = -5
		return button , "checkbutton"
	end
	Menu.Menu = _Menu
end

do--Button
	local nextName = Addon:CreateNameGenerator("Button")

	local function Button(panel, info)
		local name, isColor, handler, func
		if type(info) == "string" then
			name = info
		else
			name, isColor, handler, func = info.name, info.isColor, info.handler, info.func
		end
		
		local owner = panel.owner
		
		local b = CreateFrame("Button", nextName(), panel, "UIMenuButtonStretchTemplate")
		b:SetSize(140, 25)
		b:SetText(name or 1)
		b.text = b:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		b.text:SetDrawLayer(BACKGROUND)
		if isColor then
			b.texture = b:CreateTexture(nil, "BACKGROUND")
			b.texture:SetDrawLayer("BACKGROUND", -8)
			b.texture:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
		else
			b.texture = CreateFrame("Frame", b:GetName().."texture", b)
			b.texture:SetFrameLevel(1)
		end
		b.text:SetDrawLayer("ARTWORK", 7)
		if handler then
			b:SetScript("OnClick", function()
				func(owner.owner[handler])
			end)
		end
		b.texture:SetPoint("TopLeft", b, "BottomLeft")
		b.texture:SetPoint("TopRight", b, "BottomRight")
		b.texture:SetHeight(25)
		b.text:SetAllPoints(b.texture)
		panel.height = panel.height + 27
		b.height = 27
		local prev = panel.lastWidget
		if prev then
			b:SetPoint("TopLeft", panel.lastWidget, "BottomLeft", 0, -2)
		else
			b:SetPoint("TOPLEFT", 2, -2)
		end
		panel.lastWidget = b
		return b, "checkbutton"
	end
	Menu.Button = Button
end

do --ColorPicker(
	local nextName = Addon:CreateNameGenerator("ColorPicker")

	local function ShowColorPicker(r, g, b, a, changedCallback)
		ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = nil, nil
		ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = nil, nil, nil
		ColorPickerFrame:SetColorRGB(r,g,b)
		ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a
		ColorPickerFrame.previousValues = {r,g,b,a}
		ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = changedCallback, changedCallback, changedCallback
		ColorPickerFrame:Hide() -- Need to run the OnShow handler.
		ColorPickerFrame:Show()
	end

	local function ColorPicker(panel, info) --this thing needs help.
		local id, key = info.name,info.key
		local button = CreateFrame("Button", nextName(), panel)
		button:SetSize(24, 24)
		button:SetHitRectInsets(0,-75,0,0)
		local text = button:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		text:SetText(id)
		text:SetPoint("Left", button, "Right", 5, 0)

		local owner = panel.owner
		
		button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
		button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
		button:GetHighlightTexture():SetBlendMode("ADD")
		button:SetNormalTexture("Interface\\\BUTTONS\\WHITE8X8")
		
		local border = button:CreateTexture(nil, "OVERLAY")
		border:SetAllPoints(button)
		border:SetTexture("Interface\\BUTTONS\\UI-Quickslot2")
		border:SetTexCoord(.2,.8,.2,.8)
		button:SetScript("OnShow", function(self)
			local saved
				if panel.sets then
					saved = panel.sets[key]
				else
					saved = owner.sets[panel.id][key]
				end
			
			
			if saved then
				button:GetNormalTexture():SetVertexColor(saved.r, saved.g, saved.b, saved.a)
			end
		end)
		button:SetScript("OnClick", function(self)
			local saved
			if panel.sets then
				saved = panel.sets[key]
			else
			--	saved = owner.sets[panel.id][key]
			end
			
			local r, g, b, a = saved.r, saved.g, saved.b, saved.a
			ShowColorPicker(r, g, b, a, function(restore)
				local newR, newG, newB, newA
				if restore then
					newR, newG, newB, newA = unpack(restore)
				else
					newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB()
				end
				saved.r, saved.g, saved.b, saved.a = newR, newG, newB, newA
				self:GetScript("OnShow")(self)
				owner:Layout()
			end)
			ColorPickerFrame:EnableKeyboard(nil)
		end)
		local prev = panel.lastWidget
		if prev then
			button:SetPoint("TopLeft",  prev, "BottomLeft", 0, -2)
		else
			button:SetPoint("TOPLEFT", 2, 0)
		end
		panel.lastWidget = button
		panel.height = panel.height + 24
		button.height = 24
		button:Show()
		return button , "checkbutton"
	end
	Menu.ColorPicker = ColorPicker
end

do	--Media
	local function Media(panel, info)--panel, "Background", "Background", "GetBackground", "SetBackground"
		local name, key, handler, mediaType = info.name, info.key, info.handler, info.mediaType
		if not  panel.owner then return end
		
		local owner = panel.owner
		local button = Addon.MediaPanel:NewMediaButton(panel, name, mediaType)
		local handler = string.lower(panel.id)
		button.get = function()
			
				if panel.sets then
					return panel.sets[key]
				else
					return owner.sets[handler][key]
				end
		
		end
		button.set = function(...)
				if panel.sets then
					if panel.sets[key] ~= ... then
						panel.sets[key] = ...
						panel:Layout()
					end
				else
					owner.sets[handler][key] = ...
					owner:Layout()
				end
		end
		local medi
		button:SetScript("OnClick", function()
				medi = Addon.MediaPanel:Display(panel, button, mediaType)
				if medi and (medi.holding == button) then
					medi.get = function() return button.get() end
					medi.set = function(...) return button.set(...) end
					medi:UpdateList()
					medi:Show()
				end
			end)
			button.height = 27
			if string.lower(mediaType) == "overlay" then
				
			button.height = 80
			end
			
			button:SetScript('OnMouseWheel', function(self, direction)
				local wheel = button:GetParent():GetScript("OnMouseWheel")

				if wheel then
					wheel(panel, direction)
				end
			end)
			
			panel.height = panel.height + (55)
			panel.lastWidget = button

			panel:Render()
			
			
			button:SetHeight(button.height)
			
			button:Show()
		return button , "checkbutton"
	end
	Menu.Media = Media
end

do
	local function Panel(self, opts)
		local panelSelector = Dominos:GetOptions().PanelSelector:New(self)
		panelSelector:SetPoint('TopLeft', self, 'TopLeft', 0, 20)
		panelSelector:SetPoint('BottomRight', self, 'TopRight', -30, 0)
		panelSelector.OnSelect = function(self, id)
			self:OnShowTab(id or self.id)
		end
		self.panelSelector = panelSelector	

		self:SetPoint("TopLeft", 0, 40)
		self:SetPoint("BottomRight",0, 0)

		self.AddTab = tabbedPanel.New
		local key = string.lower(opts.name)

		self.panels = self.panels or {}
		self.panels[self.owner.id] = self.panels[self.owner.id] or {}
		
		self.panels[self.owner.id][key] = self.panels[self.owner.id][key] or self:AddTab(key)
		local panel = self.panels[self.owner.id][key]

		panel.id = key
		panel.owner = self.owner
		panel.sets = self.owner.sets[self.id][key]
		function panel:Layout()
			panel.owner[self:GetParent().id]:Layout()
		end
		
		local trace, p, d = PrepareOptions(opts.options)

		panel._tabs = panel._tabs or {}
		for i, options in pairs(p) do
			local options = Addon.Copy(options)
			local allow = GetRestrict(panel)
			if allow then
				if not panel._tabs[options.title] then
					local tab = panel:NewTab(options.title)
					tab.sets = panel.sets[string.lower(options.title)]
					function tab:Layout()
						panel:Layout()
					end
					tab.id = self.id
					tab.owner = panel
					for name, option in pairs(options.info) do
						local func = Menu[option.kind]
						if func then
							local o, type = func(tab, option)
							if o then
								tab.toggles[type] = tab.toggles[type] or {}
								tinsert(tab.toggles[type], o)
							end
						end
					end
					layout(tab)
					panel._tabs[options.title] = tab
				end
			end
		end
		panel:ShowTab(d)
	end
	Menu.Panel = Panel
end

local L

local files = {}
local ShowMenu = function (panel, level)
	if level == 1 then
		files = Addon.Profiles.GetProfiles()
		local info = L_UIDropDownMenu_CreateInfo()
		for profile, settings in pairs(files) do
			wipe(info)
			info.text = profile
			info.value = profile
			info.keepShownOnClick = true
			info.displayMode = "MENU"
			info.hasArrow = 1
			if profile == Dominos.db:GetCurrentProfile()then
				info.checked = true
			end
			L_UIDropDownMenu_AddButton(info)
		end

	elseif (level == 2) and L_UIDROPDOWNMENU_MENU_VALUE then
		local owner = panel.owner
		if (not L_UIDROPDOWNMENU_MENU_VALUE) or not files then
			return
		end
		local info = L_UIDropDownMenu_CreateInfo()
		for id, layout in pairs(files[L_UIDROPDOWNMENU_MENU_VALUE]) do
			wipe(info)
			info.text = id
			info.func = function()
				local p = Addon.Merge(layout, owner.sets, nil, {x=1, y=1, point=1})
				Addon.master.db.profile.frames[owner.id] = p
				owner.sets = p
				owner:Reload()
				owner:Layout()
				owner:Rescale()
				owner:Reposition()
				owner:Reanchor()
			end
			L_UIDropDownMenu_AddButton(info, 2, L_UIDROPDOWNMENU_MENU_VALUE)
		end
	end
	
end

local function isLeftToRight(layout, looped)
	for key, value in pairs(layout) do
		if (not looped) and (type(value) == 'table') then
			isLeftToRight(value, true) --don't modify top level settings, would alter frame position. 
		elseif looped then
			if type(value) == "table" then
				isLeftToRight(value, true)
			else
			
				if (key == 'anchor')then
					if string.find(value, 'Top') then
						layout[key] = string.gsub (value, 'Top', 'Bottom')
					elseif string.find(value, 'Bottom') then
						layout[key] = string.gsub (value, 'Bottom', 'Top')
					end
				end
				if key == "isTopToBottom" then
					layout[key] = not value
				end
				 if (key == 'justifyV') then
					if value == 1 then
						layout[key] = 3
					elseif  value == 3 then
						layout[key] = 1
					end
				 end
				if key == 'y' then
					layout[key] = -value					
				end
			end
		end
	end
end

local function FlipVertical(layout, looped)
	for key, value in pairs(layout) do
		if (not looped) and (type(value) == 'table') then
			FlipVertical(value, true) --don't modify top level settings, would alter frame position. 
		elseif looped then
			if type(value) == "table" then
				FlipVertical(value, true)
			else
				
				if (key == 'anchor')then
					if string.find(value, 'Left') then
						layout[key] = string.gsub (value, 'Left', 'Right')
					elseif string.find(value, 'Right') then
						layout[key] = string.gsub (value, 'Right', 'Left')
					end
				end
				
				if key == "isLeftToRight" then
					layout[key] = not value
				end
				
				if (key == 'justifyH') then
					if value == 1 then
						layout[key] = 3
					elseif  value == 3 then
						layout[key] = 1
					end
				end
				if key == 'x' then
					layout[key] = -value					
				end
				if key == 'spin' then
					layout[key] = -value					
				end
				if key == 'spin' then
					layout[key] = -value					
				end
			end
		end
	end
end

Addon.FlipVertical = FlipVertical

local function AddLayoutPanel(menu)
	local panel = menu:NewPanel(L.Layout)

	panel:NewSlider({
		name = 'Height',
		min = 10,
		max = 600,
		get = function(self) --Getter
			return self:GetParent().owner.sets.height
		end,

		set = function(self) --Setter
			self:GetParent().owner.sets.height = self:GetValue()
			self:GetParent().owner:Layout()
		end,
	})
	panel:NewSlider({
		name = 'Width',
		min = 10,
		max = 600,
		get = function(self) --Getter
			return self:GetParent().owner.sets.width
		end,

		set = function(self) --Setter
			self:GetParent().owner.sets.width = self:GetValue()
			self:GetParent().owner:Layout()
		end,
	})
	panel.paddingSlider = panel:NewPaddingSlider()
	panel.scaleSlider = panel:NewScaleSlider()
	
	panel.opacitySlider = panel:NewOpacitySlider()
	panel.fadeSlider = panel:NewFadeSlider()
	
		panel:NewCheckButton{
			name = "Magic Width",
			get = function(self) return panel.owner.sets.magicWidth end,
			set = function(self, enable)
				panel.owner.sets.magicWidth = self:GetChecked() or false
				panel.owner:Layout()
			end
		}	
	

	return panel
end

local function AddAdvancedPanel(menu)
	local panel = menu:NewPanel(L.Advanced)
	panel:NewClickThroughCheckbox()
	panel:NewShowInOverrideUICheckbox()
	panel:NewShowInPetBattleUICheckbox()
	
	local flipVert = Addon.Menu.Button(panel, 'Flip Vertical')
	flipVert:SetScript('OnClick', function(self)
		FlipVertical(panel.owner.sets)
		panel.owner:Layout()
	end)

	local flipHori = Addon.Menu.Button(panel, 'Flip Horizontal')
	flipHori:SetScript('OnClick', function(self)
		isLeftToRight(panel.owner.sets)
		panel.owner:Layout()
	end)

	panel:NewSlider({
		name = 'Frame Layer',
		min = 1,
		max = 100,
		get = function(self) --Getter
			self:GetParent().owner.sets.frameLayer = self:GetParent().owner.sets.frameLayer or 4
			return self:GetParent().owner.sets.frameLayer
		end,

		set = function(self) --Setter
			self:GetParent().owner.sets.frameLayer = self:GetParent().owner.sets.frameLayer or 4
			self:GetParent().owner.sets.frameLayer = self:GetValue()
			self:GetParent().owner:Layout()
		end,
	})

	panel:NewSlider({
		name = 'Frame Strata',
		min = 1,
		max = 8,
		get = function(self) --Getter
			self:GetParent().owner.sets.frameStrata = self:GetParent().owner.sets.frameStrata or 4
			return self:GetParent().owner.sets.frameStrata
		end,

		set = function(self) --Setter
			self:GetParent().owner.sets.frameStrata = self:GetParent().owner.sets.frameStrata or 4
			self:GetParent().owner.sets.frameStrata = self:GetValue()
			self:GetParent().owner:Layout()
		end,
	})

	local f =  Addon.Menu.Menu(panel, 'Copy', 'profile')
	f:SetScript('OnShow', function(self)
		if self ~= f then
			return
		end
		f:initialize()
	end)
	
	function f:initialize(level, ...)
		ShowMenu(panel, level)
		f:SetSize(277, 16)
	end
	local b = Addon.Menu.Button(panel, 'Delete Frame')
	b:SetScript('OnClick', function(self)
		if not b.clicked then
			b.clicked = true
			b:SetText('Are you sure?')
		else
			Addon.master.db.profile.units.frames[panel.owner.id] = false
			local frames = Addon.frames
			if panel.owner then
				panel.owner:Delete()
			end
			if LibStub and LibStub('LibKeyBound-1.0') then
				LibStub('LibKeyBound-1.0'):Deactivate()
			end
			Addon.pauseDelay:ResetLock(1)
		end
	end)

	local input = panel:NewTextInput{
		name = L.ShowStates,
		multiline = true,
		width = 290,
		height = 64,
		get = function() return panel.owner:GetShowStates() end,
		set = function(_, value) panel.owner:SetShowStates(value)  end
	}
	
	input.editBox:SetScript('OnEnterPressed', function(self) input:Save() self:ClearFocus() end)
	return panel
end

function Addon:CreateMenu()
	if not self.menu then
		self.menu = self.master:NewMenu(self.id)
		local panel = AddLayoutPanel(self.menu)

		if self.widgets then
			for i, b in pairs(self.widgets) do
				if b.CreateMenu and self.Menu then
					b:CreateMenu(self.menu)
				end
			end
		end
		AddAdvancedPanel(self.menu)
	end
	return self.menu
end

local menu, config
function Addon:ShowMenu()
	if not self.master:IsConfigAddonEnabled() then return end
	config = config or LoadAddOn('Dominos_Config')
	L = L or LibStub('AceLocale-3.0'):GetLocale('Dominos-Config')
	
		local menu = self.menu or self:CreateMenu()
		self.menu = menu
		menu:Hide()
		menu:SetOwner(self)
		menu:ShowPanel(L.Layout)
		menu:Show()
end
