local Report = {}

function Report:Load()
	self.panel = self.panel
	if not self.panel then
		self.size = 300
		local size = self.size
		local division = (size/(size/10))
		self.division = division
	
		local drop_handler = CreateFrame("Frame", nil, UIParent)

	
		local p = CreateFrame("Frame", nil, UIParent)
		p:SetPoint("Center", -500, 0)
		p:SetSize(size, size)
		p:Hide()
		self.panel = p

		drop_handler:SetParent(p)
		drop_handler:SetFrameLevel(p:GetFrameLevel()-1)


		drop_handler:SetBackdrop{ 
			--bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = false, tileSize = 0, edgeSize = 32, 
		  insets = { left = 0, right = 0, top = 0, bottom = 0 }
		}
		drop_handler:SetBackdropColor(0,0,0, 1)
		
		self.title = p:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
		self.title:SetPoint("Bottom", p,"Top", 0, 3)

		self.ValueBackground = p:CreateTexture(nil, "BACKGROUND")
		self.ValueBackground:SetPoint("TopRight", p, "TopLeft", 0, 15)
		self.ValueBackground:SetPoint("BottomRight", p, "BottomLeft", 0, -15)
		self.ValueBackground:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
		self.ValueBackground:SetVertexColor(0,0,0)

		self.Back = p:CreateTexture(nil, "BACKGROUND")
		self.Back:SetPoint("TopLeft", 0, 15)
		self.Back:SetPoint("BottomRight", 15, -15)
		self.Back:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
		self.Back:SetVertexColor(0,0,0)


		drop_handler:SetPoint("TopLeft", self.ValueBackground, -8, 7)
		drop_handler:SetPoint("BottomRight", self.Back, 8, -7)
		
		self:LoadLines()
		
	end
end

function Report:LoadLines()
	self.lines = {Vert = {},Hor = {},}
	local lineSize= 1
	local numLines = (self.size/10)+1
	for i = 1, numLines do
		local line = self.panel:CreateTexture(nil, "ARTWORK")
		line:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
		line:SetVertexColor(1,1,1)
		line:SetSize(lineSize, lineSize)
		local y = (self.division*(i-1))
		line:SetPoint("BottomLeft", 0, y)
		line:SetPoint("BottomRight", 0, y)
		tinsert(self.lines.Hor, line)
		line.text = self.panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
		line.text:SetPoint("Right", line, "Left", -5, 0)
		if i == (numLines/2) then
			line:SetVertexColor(0,1,1)
		end
	end
end
Report:Load()
local dots = {}
function Report:SetData(data, name)
	self:ResetData()
	local data = data[name]
	if (#data == 0) or InCombatLockdown() then
		return
	end
	self.title:SetText("Detailed Report: "..name)
	self.panel:Show()
	local _min, _max = 0, 0
	for i, value in pairs(data) do
		if value > _max then
			_max = value
		elseif value < _min then
			_min = value
		end
	end
	local _abs = ceil(max(abs(_max), abs(_min)))
	local String
	local textWidth = 0
	for i, b in pairs(self.lines.Hor) do
		i = i
		local f = -_abs
		local div = _abs/(#self.lines.Hor/2)
		local val = f + (div * i)
		local Abs = "+ "
		if val < 0 then
			Abs = "- "
		elseif val == 0 then
			Abs = "  "
		end
		local tip = ceil(math.abs(val))
		if name == "Gold" then
			if tip > 10000 then
				tip = floor(tip/10000) * 10000
			elseif tip > 1000 then
				tip = floor(tip/1000) * 1000
			end
			tip = GetCoinTextureString(tip)
		end	

		b.text:SetText(Abs..tip)
		local w = b.text:GetWidth()
		if w > textWidth then
			textWidth = w
			String = b.text
		end
	end
	if String then
		self.ValueBackground:SetPoint("Left", String, -5, 0)
	end
	for i, value in pairs(data) do
		local dot
		if not dots[i] then
			dot = self.panel:CreateTexture(nil, "ARTWORK")
			dot:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
			dot:SetVertexColor(1,0,0)
			dot:SetSize(4,4)
			dots[i] =dot
		end
		dot = dot or dots[i]
		dot:Show()
		local size = self.size
		local x =  (((i-1)/(#data)) * size)
		local y = (value * (size/2)) /_abs
		dot:SetWidth(size/#data)
		dot:SetPoint("Left",x,y)
	end
end

function Report:ResetData()
	self.panel:Hide()
	for i, b in pairs(dots) do
		b:Hide()
	end
end



local Goalie = CreateFrame("Frame")

function Goalie:Load(parent)
	parent.sets = parent.sets or {}
	parent.sets.Goal = parent.sets.Goal or {}
	parent.Flasher = parent.Flasher or parent:CreateTexture(nil, "ARTWORK")
	parent.Flasher:SetTexture("Interface\\FullScreenTextures\\LowHealth")
	parent.Flasher:SetAllPoints(parent)
	parent.Flasher:SetAlpha(0)
end

function Goalie:SetGoal(parent, name, goal)
	local data = parent.sets.currency.Goal
	data[name] = goal
end

function Goalie:UpdateGoal(parent, name, currentValue)
	if not parent.sets.Goal then
		return
	end
	local data = parent.sets.Goal[name]
	if data and (currentValue >= data) then
		Goalie:GoalMet(parent)
	else
		parent.Flasher:SetAlpha(0)
	end
end


local Max = 1
local Half = Max/2

local Time 
function Goalie:GoalMet(parent)
	Time = Time or GetTime()
	local t =  GetTime() - Time
	local alpha = (math.abs(Half-t)/Max)
	parent.Flasher:SetAlpha(alpha)
	if t > Max then
		Time = GetTime()
	end
end

function Goalie:DisplayGoal(parent, name, currentValue)
	parent.sets.Goal = parent.sets.Goal or {}
	local data = parent.sets.Goal[name]
	if (not data) then return end
	if data and (currentValue >= data) then
		print("Goal Met! ".. name..": "..data)
		Goalie:ClearGoal(parent, name)
	end
end

function Goalie:ClearGoal(parent, name)
	parent.sets.Goal[name] = nil
end

local Addon = _G[...]
local modName = ...

local widget = Addon:NewWidget('currency', 'Frame')
widget.Events = {
	["PLAYER_MONEY"] = 1,
	["ARTIFACT_UPDATE"] = 1,
	["ARTIFACT_XP_UPDATE"] = 1,
	["CURRENCY_DISPLAY_UPDATE"]  = 1,
	["UPDATE_FACTION"] = 1,
	["LFG_BONUS_FACTION_ID_UPDATED"] = 1,
}

function widget:New(parent)
    if parent.id ~= "player" then
        return
    end
	local name = ('%s_%s_currencyBar'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local bar = self:Bind(CreateFrame('Frame', name, parent.box))
	bar.owner = parent
	bar.watchIndex = 1


	bar.displayIcon = bar:CreateTexture(nil, "ARTWORK")
	bar.displayIcon:SetSize(15, 15)
	bar.displayIcon:SetPoint("Right")
	
	bar.count = bar:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	bar.count:SetHeight(15)
	bar.count:SetJustifyH("Right")
	bar.count:SetPoint("Right", bar.displayIcon , "Left",-3,0)

	bar.text = bar:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	bar.text:SetHeight(15)
	bar.text:SetJustifyH("LEFT")
	bar.text:SetPoint("Right", bar.count, "Left", -7, 0)

	
	bar.drop = bar.drop or CreateFrame('Frame', nil, bar)
	bar.drop:SetAllPoints(bar)
    bar:SetSize(30, 15)
	
	
	bar.mouseManager = CreateFrame("Frame", nil, bar)
	bar.mouseManager:SetPoint("TopLeft", bar.text)
	bar.mouseManager:SetPoint("BottomRight", bar.displayIcon)
	bar.mouseManager:SetScript("OnEnter", self.OnEnter)
	bar.mouseManager:SetScript("OnLeave", self.OnLeave)

	bar.mouseManager:SetScript("OnMouseWheel", self.OnMouseWheel)
	
	Goalie:Load(bar)
	
	return bar
end

local function round(val, decimal)
  if (decimal) then
    return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
  else
    return math.floor(val+0.5)
  end
end

local currentTrack, currentInfo
function widget:OnEnter()
	local parent = self:GetParent()
	if currentTrack then
		local name = currentTrack
		
		local gain = " Gained"
		
		if currentInfo then
			gain = ""
		end
		
		GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
		GameTooltip:SetText(name..gain)
		
		local tip = math.abs(parent.sets.ThisSession[name])
		
		local Abs = "+ "
		if parent.sets.ThisSession[name] < 0 then
			Abs = "- "
		elseif parent.sets.ThisSession[name] == 0 then
			Abs = "  "
		end
		
		if name == "Gold" then
			tip = GetCoinTextureString(tip)
		end
		
		if not currentInfo then
			GameTooltip:AddDoubleLine("This Session:",Abs..tip)
		end
		
				
		local Abs = "+ "
		if parent.sets.lifeTime[name] < 0 then
			Abs = "- "
		elseif parent.sets.lifeTime[name] == 0 then
			Abs = "  "
		end
		
		local tip = math.abs(parent.sets.lifeTime[name])
		if name == "Gold" then
			tip = GetCoinTextureString(tip)
		end	
		if not currentInfo then
			GameTooltip:AddDoubleLine("Life Time:", Abs..tip)
		end
		if currentInfo then
			for i, info in pairs(currentInfo) do
				
				GameTooltip:AddDoubleLine(info.slot, round(info.value, 1))
			end
		end
		
		
		if not currentInfo then

			parent.sets.lifeTime_Track[name] = parent.sets.lifeTime_Track[name] or {}
			if parent.sets.advLog then
				Report:SetData(parent.sets.lifeTime_Track, name)
			end
			if parent.sets.Goal[currentTrack] then
			GameTooltip:AddDoubleLine("Goal:",  tonumber(parent.sets.Goal[currentTrack]))
			end
		end
				GameTooltip:Show()

	end
end

function widget:OnLeave()
	GameTooltip:Hide()
	if self:GetParent().sets.advLog then
		Report:ResetData()
	end
end
	
function widget:OnMouseWheel(delta)
	local self = self:GetParent()
	local numWatched = self:GetNumWatched()
		
	if self.sets then
		if not self.sets.watchIndex then
			self.sets.watchIndex = 1
		end
		self.sets.watchIndex = self.sets.watchIndex + delta
		
		if self.sets.watchIndex > numWatched then
			self.sets.watchIndex = 1
		elseif self.sets.watchIndex < 1 then
			self.sets.watchIndex = numWatched
		end
	end
	if MouseIsOver(self.mouseManager) then
		self:Update()
		self.mouseManager:GetScript("OnEnter")(self.mouseManager)
	end
end

widget.defaults = {
	alpha = 100,
	showBG = false,
	isTopToBottom = false,
	enable = true,
	anchor = 'BottomLeft',
	x = 12,
	y = 6,
	scale = 64,
	width = 50,
	color = {
		r = 1,
		g = 1,
		b = 1,
		a = 1,	
	},
	watchIndex = 1,
	lifeTime = {},
	ThisSession = {},
	lifeTime_Track = {},

}

function widget:Load()
    if self.owner.id ~= "player" then
        return
    end

	self:EnableMouse(false)
	self.noMouse = true
	self.id = self.owner.id
end

function widget:Layout()
	local sets = self.sets
	if sets.enable ~=true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end
	self:ClearAllPoints()
	local x = sets.x/(sets.scale/100)
	local y = sets.y/(sets.scale/100)
	self:SetPoint(sets.anchor, self:GetParent(), x, y)
	
	self:SetWidth(sets.width)
	
	if sets.showBG then
		self.drop:SetBackdrop(self.bg)
		self.drop:SetBackdropColor(self.sets.color.r, self.sets.color.g, self.sets.color.b, self.sets.color.a)
	else
		self.drop:SetBackdrop(nil)
	end
end

function widget:UpdateTracked(name, currencyValue,info)
	local ThisSession = self.sets.ThisSession
	local lifeTime = self.sets.lifeTime
	local lifeTime_Track = self.sets.lifeTime_Track
	lifeTime_Track[name] = lifeTime_Track[name] or {}

	self.PrevCount = self.PrevCount or {}
	
		if not self.PrevCount[name] then
			ThisSession[name] = 0
			lifeTime[name] = lifeTime[name] or 0
			self.PrevCount[name] = currencyValue
		end
		
		
		if self.PrevCount[name] ~= currencyValue then

			ThisSession[name] = ThisSession[name] + (currencyValue - self.PrevCount[name])
			lifeTime[name] = lifeTime[name] + (currencyValue - self.PrevCount[name])
			
			--if ThisSession[name] ~= 0 then
				tinsert(lifeTime_Track[name], ThisSession[name])
			--end
		end
		self.PrevCount[name] = currencyValue	
end

local durability = {}



local function DurabilityFrame_GetAlerts()
	local damage, maximum = 0, 0
	wipe(durability)
	for index, value in pairs(INVENTORY_ALERT_STATUS_SLOTS) do
		local current, base  = GetInventoryItemDurability(index)
		if current then
			tinsert(durability, {slot = value.slot, value = (current/base) * 100})
			damage = damage + current
			maximum = maximum + base
		end
	end
	return round((damage/maximum) * 100, 1) , durability
end



local track = {
	{
		name = "Armor",
		icon = "Interface\\COMMON\\FavoritesIcon",
		value = function()
			local damage, maximum = 0, 0
			wipe(durability)
			for index, value in pairs(INVENTORY_ALERT_STATUS_SLOTS) do
				local current, base  = GetInventoryItemDurability(index)
				if current then
					tinsert(durability, {slot = value.slot, value = (current/base) * 100})
					damage = damage + current
					maximum = maximum + base
				end
			end
			return round((damage/maximum) * 100, 1) , durability
		end
	},

	{
		name = "Artifact Points",
		value = function()
			local _, _, _, texture, num_points = C_ArtifactUI.GetEquippedArtifactInfo()
			return num_points , nil, texture
		end
	},
	
	{
		name = "Gold",
		value = function()
			return GetCoinTextureString(GetMoney())
		end
	},

}
local GetTokens
do
	local tokens = {}
	function GetTokens(index)
		wipe(tokens)
		for i=1, GetNumWatchedTokens() do
			local name, count, icon = GetBackpackCurrencyInfo(i)
			if name then
				tinsert(tokens, {name = name, value = count, icon = icon, info = nil})
			end
		end

		for i, info in pairs(track) do
			local current, inf, texture  = info.value()
			tinsert(tokens, {name = info.name, value = current, icon = info.icon or texture, info = inf })
		end
		
		return tokens
	end
end
local function GetTokenInfo(index)
	local tokens  = GetTokens()
	local token = tokens[index]
	if not token then
		return
	end
	return token.name, token.value, token.icon, token.info
end

function widget:GetNumWatched()
	return GetNumWatchedTokens() + #track
end

function widget:Update(elapsed)
    if self.owner.id ~= "player" then
        return
    end
	if (self.sets.enable ~= true) or self.noUpdate then
		return
	end
	
	local watched = self.sets.watchIndex
	local autoWatch = watched
	
	local numWatched = GetNumWatchedTokens()
	
	self.watching = self.watching or {}
	
	
	for i, token in pairs(GetTokens()) do
		local name, value, icon, info = token.name, token.value, token.icon, token.info
		
		local prevValue = self.watching[name]

		if prevValue and (value ~= prevValue) then
			autoWatch = i
			self.sets.watchIndex = i
		end
		if name then
			self.watching[name] = value
		end
	end

	local name, count, icon, info = GetTokenInfo(autoWatch)
	if not name then
		return
	end
	currencyValue = tonumber(count) or 0
	local ThisSession = self.sets.ThisSession
	local lifeTime = self.sets.lifeTime
	local lifeTime_Track = self.sets.lifeTime_Track
	lifeTime_Track[name] = lifeTime_Track[name] or {}

	self.PrevCount = self.PrevCount or {}
			currentTrack = name
			currentInfo = info

	if (self.PrevName ~= name) or (self.PrevCount[name] ~= currencyValue) then
		self:UpdateTracked(name, currencyValue, info)
		currentTrack = name
		currentInfo = info
		if self.text then
			self.text:SetText(name)
		end
		self.count:SetText(count)
		if icon == "" then
			self.displayIcon:Hide()
		else
			self.displayIcon:SetTexture(icon)
			self.displayIcon:Show()
		end
	end
	currentTrack = name 
	Goalie:UpdateGoal(self, name, currencyValue)
	self.PrevName = name
	
	if MouseIsOver(self)then
		Goalie:DisplayGoal(self, name, currencyValue)
	end	
	
end

widget.bg = {
	bgFile = 'Interface\\Tooltips\\UI-Tooltip-Background',
	insets = {left = 0, right = 0, top = 0, bottom = 0},
	tile = false,
}

function widget:GetRestrictions()
	return "player", true
end

widget.Options = {
					{
                        name = 'Show Details',
                        kind = 'CheckButton',
                        key = 'advLog',
						panel = "advanced",
                    },
                    {
                        name = 'Auto Track',
                        kind = 'CheckButton',
                        key = 'AutoWatch',
						panel = "basic",
                    },

					-- [[
                    {
                        name = 'Set Goal',
                        kind = 'EditBox',
                        key = function(self)
							if currentTrack and self.sets.currency.Goal and self.sets.currency.Goal[currentTrack] then
								return self.sets.currency.Goal[currentTrack], currentTrack
							else
								return "", currentTrack
							end
						end,
						setter = function(self, text)
							if currentTrack then
								Goalie:SetGoal(self, currentTrack, tonumber(text))
							end
						end,
						panel = "goal",
                    },
					--]]

                    {
                        name = 'Backdrop',
                        kind = 'CheckButton',
                        key = 'showBG',
						panel = "advanced",
                    },
                    {
                        name = 'Backdrop Color',
                        kind = 'ColorPicker',
                        key = 'color',
						panel = "advanced",
                    },


        }
		