
local Addon = _G[...]
local modName = ...
local title = "Background"


local widget = Addon:NewWidget(title, "Frame")

widget.defaults = {
	visibility = {
		border = {
			color = {
				a = 0.5,
				b = 0,
				g = 0,
				r = 0,
			},
			file = "Blizzard",
			thickness = 10,
		},
		background = {
			enable = true,
			file = "Raid",
			color = {
				a = 0.5,
				b = 0,
				g = 0,
				r = 0,
			},
		},
	},
	basic = {
		position = {
			y = -14,
			x = 4,
			frameLevel = 3,
			anchor = "TopLeft",
			frameStrata = 2,
		},
		size = {
			height = 15,
			enable = true,
			width = 116,
		},
	},
}


function widget:New(parent)	
--	local nil = ('%s_%s_'..title):format(modName, parent.id)
--	if _G[name] then return _G[name] end

	local bar = self:Bind(CreateFrame('Frame', nil, parent.box))
	bar.owner = parent
	bar:SetFrameLevel(1)

	bar.drop = bar.drop or CreateFrame('StatusBar', nil, bar)
	bar.drop:SetMinMaxValues(0,1)
	bar.drop:SetValue(1)
	bar.drop:SetAllPoints(bar)
	
	local v = bar:GetFrameLevel()-2
	if v < 1 then v = 1 end
	
	bar.drop:SetFrameLevel(v)	
	bar.owner = parent
	bar.title = title
	bar.handler = parent.id
	return bar
end

function widget:Free()
	
end

function widget:Layout()
	if self.sets.basic.size.enable ~= true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end

	self:Update()
	self:Reposition()	
	self:Resize()
	self:SetVisibility()
end


function widget:Reposition()
	local position = self.sets.basic.position
	local size = self.sets.basic.size
	
	self:ClearAllPoints()
	self:SetPoint(position.anchor, self:GetParent(), position.x, position.y)
	
	local lay = Addon.layers[position.frameStrata]
	self:SetFrameStrata(lay)
	self:SetFrameLevel(position.frameLevel)	
	
end

local function LookForSets(frame)
	if not frame:GetParent() then
		return nil, "Dead End"
	elseif frame:GetParent().sets then
		return frame:GetParent().sets
	else
		return LookForSets(frame:GetParent())
	end
end

function widget:Resize()
	local size = self.sets.basic.size
	
	local width, height = (size.width), (size.height)
	self:SetHeight(height)
	
	
	local set = LookForSets(self)
	if set then
		local w = set.width
		if set.magicWidth then
			local d = w - 191
			self:SetWidth((size.width  + d))
		else
			self:SetWidth(width)
		end
	else
		self:SetWidth(width)
	end
end





function widget:SetVisibility()
	local visibility = self.sets.visibility
	local background = visibility.background
	local border = visibility.border

	local pad = background.padding
	
	local BG = self:GetMediaPath("statusbar", background.file)
	local brd = self:GetMediaPath("border", border.file)
	
	if not background.enable then
		BG = ""
	end
	if not border.enable then
		brd = ""
	end
	self.drop:SetBackdrop({ 
		bgFile = BG, 
		edgeFile = brd,
		tile = false,
		edgeSize = border.thickness, 
		insets = { left = pad, right = pad, top = pad, bottom = pad }
	})

	do
		local color = background.color
		self.drop:SetBackdropBorderColor(color.r, color.g, color.b, color.a)
	end

	do
		local color = background.color
		self.drop:SetBackdropColor(color.r, color.g, color.b, color.a)
	end
end



function widget:Update()
	if self.noUpdate then
		return
	end
	if self.OnUpdate then
		self:OnUpdate()
	end
end


function widget:GetMediaPath(kind, fileName)
	if Addon.lib then
		return Addon.lib:Fetch(kind, fileName)
	end
end

function widget:OnUpdate()
	local unit = self.owner.id
	
	local unit = self.owner.id
	
	local color = self.sets.visibility.background.color
	local r, b, g, a = 0, 1, 0, 1
	local unit = self.owner.id

	local classification = UnitClassification(unit);

	
	if self.sets.visibility.background.targetReaction == true then
		if ( classification == "minus" ) then
			r, b, g, a= 0, 0, 0, 0
		elseif UnitIsUnit("player", unit) then
			r, b, g, a = color.r, color.g, color.b, color.a
		elseif( not UnitIsFriend(unit, "player") and UnitPlayerControlled(unit) ) then
			if( UnitCanAttack("player", unit) ) then
				r, b, g = .9, 0, 0
			else
				r, b, g = .6, .2, .2
			end
		elseif( UnitReaction(unit, "player") ) then
			local reaction = UnitReaction(unit, "player")
			if ( not UnitPlayerControlled(unit) and UnitIsTapDenied(unit) ) then
				g, g, b, a = .5, .5, .5, .5
			elseif( reaction > 4 ) then
				r, b, g = 0, 1, 0
			elseif( reaction == 4 ) then
				r, b, g = .93, .93, 0
			elseif( reaction < 4 ) then
				r, b, g = .9, 0, 0
			end
		end
		self.drop:SetBackdropColor(r, b, g, a)
	end
end

widget.Options = {
	{
		name = "Basic",
		kind = "Panel",
		key = "basic",
		panel = "Basic",
		options = {
			{
				name = 'Enable',
				kind = 'CheckButton',
				key = 'enable',
				panel = "size",
			},

			{
				name = 'Height',
				kind = 'Slider',
				key = 'height',
				min = 10,
				max = 250,
				panel = 'size',
			},
			{
				name = 'Width',
				kind = 'Slider',
				key = 'width',
				min = 10,
				max = 250,
				panel = 'size',
			},


			{
				name = 'X Offset',
				kind = 'Slider',
				key = 'x',
				panel = 'position',
				min = -400,
				max = 400,
			},
			{
				name = 'Y Offset',
				kind = 'Slider',
				key = 'y',
				panel = 'position',
				min = -400,
				max = 400,
			},
			{
				name = 'Anchor',
				kind = 'Menu',
				key = 'anchor',
				panel = 'position',
				table = {
					'TopLeft',
					'Top',
					'TopRight',
					'Right',
					'BottomRight',
					'Bottom',
					'BottomLeft',
					'Left',
					'Center',
				},
			},
			{
				name = "Frame Level",
				kind = "Slider",
				key = "frameLevel",
				panel = 'position',
				min = 1,
				max = 100,
			},
			{
				name = "Frame Strata",
				kind = "Slider",
				key = "frameStrata",
				panel = 'position',
				min = 1,
				max = 8,
			},

		}
	},	
	{
		name = "visibility",
		kind = "Panel",
		key = "visibility",
		panel = "visibility",
		options = {
			
			{
				name = 'Enable',
				kind = 'CheckButton',
				key = 'enable',
				panel = "background",
			},
			{
				name = 'Reaction Colored',
				kind = 'CheckButton',
				key = 'targetReaction',
				panel = "background",
			},
			
			
			{
				name = 'Texture',
				kind = 'Media',
				key = 'file',
				mediaType = 'Statusbar',
				panel = "background",
			},
			{
				name = 'Background Color',
				kind = 'ColorPicker',
				key = 'color',
				panel = 'background',
			},

			
			
			{
				name = 'Texture',
				kind = 'Media',
				key = 'file',
				mediaType = 'Border',
				panel = "border",
			},
	
			{
				name = 'Enable',
				kind = 'CheckButton',
				key = 'enable',
				panel = "border",
			},
			{
				name = 'Background Color',
				kind = 'ColorPicker',
				key = 'color',
				panel = 'border',
			},
			{
				name = 'Thickness',
				kind = 'Slider',
				key = 'thickness',
				panel = "border",
				min = 1,
				max = 32,
			},

		}
	},

	}