local Addon = _G[...]
local pieces = {}

local onEnter = function(self)
	self.sets.alpha = self.sets.alpha or 100
	self:SetAlpha(self.sets.alpha/100 or 1)
end

local onLeave = function(self)
	self.sets.fadeAlpha = self.sets.fadeAlpha or 100
	self:SetAlpha(self.sets.fadeAlpha/100 or 1)
end

local basics = {
	frameStrata = 1,
	frameLevel = 2,
	alpha = 100,
	fadeAlpha = 100,
}

--Basics that all widgets share
local function Hook(frame)


	if not frame.hooked then
		frame.hooked = true
	else
		return
	end
	local def = frame.defaults
	if def then
		for i, b in pairs(basics) do
			def[i] = def[i] or b
			frame.sets[i] = frame.sets[i] or b
		end
	end

	if frame:HasScript("OnEnter") and frame.sets then
		local enter = frame:GetScript("OnEnter")
		if enter then
			frame:HookScript("OnEnter", onEnter)
		else
			frame:SetScript("OnEnter", onEnter)
		end
	end
	if frame:HasScript("OnLeave") and frame.sets then

		local leave = frame:GetScript("OnLeave")
		if leave then
			frame:HookScript("OnLeave", onLeave)
		else
			frame:SetScript("OnLeave", onLeave)
		end
	end
	
	hooksecurefunc(frame, "Layout", function(self)
		if (not self.sets) or InCombatLockdown() then
			return
		end
		--local Strata = Addon.layers[ ceil(self.sets.frameStrata or 1)] or "MEDIUM"
		
		--self:SetFrameStrata(Strata)
		--self:SetFrameLevel(ceil(self.sets.frameLevel)+1)
		
		if self.drop then
		--	self.drop:SetFrameStrata(Strata)
		--	self.drop:SetFrameLevel(ceil(self.sets.frameLevel))
		end
		self:SetAlpha(self.sets.fadeAlpha/100 or 1)
		if self.key == "focus" then
			local s = true
			if self.sets.disableMouse and self.sets.disableMouse == true then
				s = false
			end
			self:EnableMouse(s)
		else
			self:EnableMouse(false)
		end

	end)
end

local allWidgets = {}
Addon.allWidgets = allWidgets

function Addon:Load()
	self.widgets = self.widgets or {}
--	if #self.widgets == 0 then
		for i, info in pairs(Addon.Copy(pieces)) do
			local allow = true
			if info.func.GetRestrictions then
				local id, include = info.func.GetRestrictions()
				if (include == true) then
					if id ~= string.lower(self.id) then
						allow = false
					end
				elseif include == false then
					if id == string.lower(self.id) then
						allow = false
					end
				end
			end
			if allow then
				local widget, key = self[info.id] or info.func:New(self), info.id
				if widget then
					widget.key = key
					if widget.defaults then
						self.sets[key] = self.sets[key] or {}
						self.check(widget.defaults, self.sets[key])
						widget.sets = self.sets[key]
						self[key] = widget
						tinsert(allWidgets, widget)
					end
					widget.ID = self.id
					tinsert(self.widgets, widget)
				end
			end
		end
	--end
	for id, widget in pairs(self.widgets) do
		if widget.Load then
			widget:Load()
		end
	end
end

local globals = {
--[[
	advanced = {
		{
			name = "Frame Level",
			kind = "Slider",
			key = "frameLevel",
			min = 1,
			max = 100,
		},
		{
			name = "Frame Strata",
			kind = "Slider",
			key = "frameStrata",
			min = 1,
			max = 8,
		},
		{
			name = 'Enable',
			kind = 'CheckButton',
			key = 'enable',
			index = 1,
		},
		{
			name = 'Opacity',
			kind = 'Slider',
			key = 'alpha',
			min = 0,
			max = 100,
		},
		{
			name = 'Faded Opacity',
			kind = 'Slider',
			key = 'fadeAlpha',
			min = 0,
			max = 100,
		},
	},
--]]
	}

local overlay = {
	advanced = {
		{
			name = "Frame Level",
			kind = "Slider",
			key = "frameLevel",
			min = 1,
			max = 100,
		},
		{
			name = "Frame Strata",
			kind = "Slider",
			key = "frameStrata",
			min = 1,
			max = 8,
		},
		{
			name = 'Enable',
			kind = 'CheckButton',
			key = 'enable',
			index = 1,
		},
	},
}

function AddGlobals(self)
	if self.ADDED then
		return
	end
	local options = Addon.Copy(self.Options)
	local def = self.defaults

	local k = globals
	if self.title == "overlay" then
		k = overlay
	else
		if def then
			def.frameLayer = 1
			def.frameStrata = 2
			self.sets.frameLayer = self.sets.frameLayer or def.frameLayer
			self.sets.frameStrata = self.sets.frameStrata or def.frameStrata
		end
	end
	
	for title, menu in pairs(k) do
		for i, option in pairs(menu) do
			option.panel = title
			if option.index then
				tinsert(options, option.index, option)
			else
				tinsert(options, option)
			end
		end
	end
	self.ADDED = true
	return options
end

local base = {
	CreateMenu = function(self, menu)
		if self.Options then
			local options = AddGlobals(self)
			Addon.Menu.NewMenu(self, menu, string.lower(self.title), Addon.Copy(options))
		end
	end,
	GetRestrictions = function() end
}

local template = {}

function template:GetDefault(setting)
	local defs = Addon.Copy(self.defaults)
	return defs[setting]
end

function template:GetSettings()
	return self.sets or Addon.Copy(self.defaults)
end

function template:GetSetting(setting)
	local sets = self:GetSettings()
	local defs = Addon.Copy(self.defaults)
	sets[setting] = sets[setting] or defs[setting]
	return sets[setting]
end

function template:UpdateSetting(option, keys, value)
	local opt = self.sets[option]
	if not opt then
		return
	end
	if type(keys) == "table" then
		for i = 1 , #keys-1 do
			opt = opt[key[i]]
		end
		local key = keys[#keys]
		if value and opt[key] ~= value then
			opt[key] = value
			self:Layout()
		else
			return opt[key]
		end
	else
		if value and opt[keys] ~= value then
			opt[keys] = value
			self:Layout()
		else
			return opt[keys]
		end
	end




end

function template:GetMediaPath(mediaType, mediaName)
	if Addon.lib then
		self.sets[mediaType.."_mediaPath"] = Addon.lib and Addon.lib:Fetch(mediaType, mediaName)
	else
		return self.sets[mediaType.."_mediaPath"]
	end
	return (Addon.lib and Addon.lib:Fetch(mediaType, mediaName))
end

function Addon:NewWidget(name, frameType, newTemplate)
	local widget = Addon.master:CreateClass(frameType)
	widget.title = name
	widget.owner = self:GetParent()
 	tinsert(pieces, {id = string.lower(name), func = widget})
	Addon.Blend(Addon.Copy(base), widget)
	if newTemplate then
		Addon.Blend(Addon.Copy(Addon._TEMPLATE), widget)
	end
	Addon.Merge(Addon.Copy(template), widget)
 	return widget
end

function Addon:UpdateWidgets()
	if self.widgets then
		for id, widget in pairs(self.widgets) do
			if widget.UpdateLayout then
				widget:UpdateLayout()
			end
			if widget.Layout then
				Hook(widget)
				widget:Layout()
			end
			if widget.Update then
				widget:Update()
			end
		end
	end
end

local templates = {}
Addon.templates = templates

function Addon:NewTemplate(name)
	templates[name] = {}
	return templates[name]
end

function Addon:GetTemplate(template, name)
	local temp = templates[template]
	if temp then
		return Addon.Merge(CopyTable(templates[template]), Addon:NewWidget(string.lower(name), template))
	else
		print("Template does not exist: "..template)
	end
end
